#include <stdio.h> 
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <stdlib.h>

/*****************************************/
/*                                       */
/* Nils-SHELL                            */
/* My own Shell                          */
/* Calls programms etc.                  */
/*                                       */
/* Author: Nils Duschner                 */
/* Date: 2009/04/15                      */
/*****************************************/

#define MAXLINE 255
#define MAXPARAMS 10


void callchild(char* command, char** params);
int inShell(char* command,int parc,char** argv);
int parseParams(int parc,char** params);
void version();
void help();

int main(int argc,char** argv){
  char linetype[MAXLINE]= {0};
  char* command = NULL;
  char* param=NULL;
  char* params[MAXPARAMS+1] = {NULL};
  char hostname[20];
  pid_t thispid;
  int i=0;
  size_t len=0;
  
  if(argc > 1 && argv[1][0] == '-' && argv[1][1] == 'v') version();
  else if(argc > 1){ help(); exit(0); }
  gethostname(hostname,20);
  while(printf("%s@%s:%s$ ",getenv("USER"),hostname,getcwd(NULL,0)) && fgets(linetype,MAXLINE,stdin) != NULL){
    len = strlen(linetype); linetype[len-1] = '\0';
    command = strtok(linetype," ");
    if(command != NULL){
    for(i=0;i<MAXPARAMS;i++) params[i] = NULL;
    i=1;
    params[0] = command;
    while((param = strtok(NULL," ")) != NULL && i<MAXPARAMS){
      params[i++] = param;
    }
    parseParams(i,params);
    if(inShell(command,i,params) == 0){
      if((thispid = fork()) == 0){
        if(i <= 1) params[0] = NULL;
        callchild(command,params);
        exit(0);
      }
      else{
        waitpid(thispid,NULL,0);
      }
    }
  
  }}
  printf("\n");
  return 0;
}
/* function: callchild */
/* calls the actual "programm" which should be executed */
/* return nothing */
/* char* command (in): the programm to be executed */
/* char** params (in): the params to programm in execvp format */


void callchild(char* command,char** params){
  execvp(command,params);
  if(errno != 0) perror("Child Error");
  errno=0;
  return;
}

/* Function: inShell */
/* Figure out wether command should be executed "inShell" rather than execv */
/* returns 0 if command needs to be executeded as child, otherwise 1 */
/* char* command (in): which command was "typed" */
/* int parc (in): param counter */
/* char** params (in): array of parameters */


int inShell(char* command,int parc,char** params){
  if(strcmp("exit",command) == 0) exit(0);
  else if(strcmp("cd",command) == 0){
    chdir(params[1]);
    if(errno!=0) perror("Command error");
    errno=0;
    return 1;
  }
  else if(strcmp("help",command) == 0){
    help();
    return 1;
  }
  return 0;
}

/* Function parseParams */
/* Substitute Variables (from ENV) by their actual value */
/* returns amount of parameters, substituted */
/* int parc (in): param counter */
/* char ** params (in/out): the params */

int parseParams(int parc,char** params){
  int x,c=0;
  char* var;
  for(x=0;x<parc;x++){
    if(params[x] != NULL && params[x][0]=='$'){
        var = getenv(&params[x][1]);
        if(var != NULL){
          params[x] = var;
          c++;
        }
      }
    }
  return c;
}

void version(){
  printf("This is NSHELL in Version 1.0\n\nAuthor: Nils Duschner\nEmail: nils.duschner@fh-osnabrueck.de\n");
  help();
  exit(0);
}

void help(){
  printf("This Version of NSHELL does NOT support any parameters\n");
}
